from uuid import uuid4
from selenium.webdriver.remote.webdriver import WebDriver
from koverj.common.client.locators_life_cycle import LocatorsLifecycle
from koverj.common.model.locator import Locator


class KoverjSelenium:
    def __init__(self):
        self.lifecycle = LocatorsLifecycle()

    def setup(self, driver: WebDriver):
        uuid = str(uuid4())
        self.lifecycle.storage.locators.append(
            Locator(url=driver.current_url, locator="some locator here", uuid=uuid)
        )



from uuid import uuid4
from selene.api import browser
from koverj.agent.selenium.webdriver_locator_listener import WebDriverLocatorListener
from koverj.common.client.locators_life_cycle import LocatorsLifecycle
from koverj.common.model.locator import Locator


class KoverjSeleneListener(WebDriverLocatorListener):
    def __init__(self):
        super().__init__()
        self.lifecycle = LocatorsLifecycle()

    def setup(self):
        current_url = browser.driver().current_url
        uuid = str(uuid4())
        self.lifecycle.storage.locators.append(
            Locator(url=current_url, locator="some locator here", uuid=uuid)
        )



from typing import List
from uplink import Body
from dataclasses import dataclass
from koverj.common.model.locator import Locator


@dataclass(frozen=True)
class TestResult(Body):
    buildId: int
    testName: str
    locators: List[Locator]

    def __repr__(self):
        locators_repr = "\n\t\t".join([str(loc) for loc in self.locators])
        return f"\n{self.__class__.__name__}(" \
               f"\n\tbuildId={self.buildId}" \
               f"\n\ttestName={self.testName}" \
               f"\n\tlocators=[" \
               f"\n\t\t{locators_repr}" \
               f"\n)"



from dataclasses import dataclass


@dataclass(frozen=True)
class Locator:
    url: str
    locator: str
    subject: str = ""
    uuid: str = ""
    parent_uuid: str = "constant"
    # TODO no solution yet to get parent UUID in find().find() cases



from uuid import uuid5, NAMESPACE_DNS
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support.abstract_event_listener import AbstractEventListener
from koverj.common.client.locators_life_cycle import LocatorsLifecycle
from koverj.common.model.locator import Locator


class WebDriverLocatorListener(AbstractEventListener):
    def __init__(self, lifecycle: LocatorsLifecycle = None):
        super().__init__()
        self.lifecycle = lifecycle or LocatorsLifecycle()

    def before_navigate_to(self, url, driver):
        pass

    def after_navigate_to(self, url, driver):
        pass

    def before_navigate_back(self, driver):
        pass

    def after_navigate_back(self, driver):
        pass

    def before_navigate_forward(self, driver):
        pass

    def after_navigate_forward(self, driver):
        pass

    def before_find(self, by, value, driver: WebDriver):
        locator_repr = f"(By={by}, Value='{value}')"
        uuid = str(uuid5(NAMESPACE_DNS, name=f"{driver.current_url}{locator_repr}"))
        self.lifecycle.storage.locators.append(
            Locator(url=driver.current_url, locator=value, uuid=uuid)
        )

    def after_find(self, by, value, driver):
        pass

    def before_click(self, element, driver):
        pass

    def after_click(self, element, driver):
        pass

    def before_change_value_of(self, element, driver):
        pass

    def after_change_value_of(self, element, driver):
        pass

    def before_execute_script(self, script, driver):
        pass

    def after_execute_script(self, script, driver):
        pass

    def before_close(self, driver):
        pass

    def after_close(self, driver):
        pass

    def before_quit(self, driver):
        pass

    def after_quit(self, driver):
        pass

    def on_exception(self, exception, driver):
        pass



from uplink import Consumer, post, Body, json
from requests.models import Response


class KoverjService(Consumer):
    @json
    @post("/locators")
    def post_locators(self, test_result: Body) -> Response:
        """post locators to server"""""



from pathlib import Path
from configparser import ConfigParser, ExtendedInterpolation


class ProjectConfig:
    config = ConfigParser(interpolation=ExtendedInterpolation())
    CONFIG_DIR = Path(__file__).resolve().parent
    config.read(CONFIG_DIR.joinpath("env.cfg"))

    def __init__(self, env: str = "DEFAULT"):
        self.use_koverj = self.config.getboolean(env, "use_koverj")
        self.log_locators = self.config.getboolean(env, "log_locators")
        self.service_url = self.config.get(env, "service_url")



import logging

from koverj.common.config.project_config import ProjectConfig
from koverj.common.model.test_result import TestResult
from koverj.common.service.koverj_service import KoverjService


class KoverjClient:

    def __init__(self, config: ProjectConfig = None):
        self.config = config or ProjectConfig()
        self.service = KoverjService(base_url=self.config.service_url)

    def send_test_result(self, test_result: TestResult):
        try:
            response = self.service.post_locators(test_result)
            if response.status_code != 200:
                logging.debug(response.text)
                raise RuntimeError(f"Locators not saved: {response.status_code}")
        except Exception as e:
            logging.error(e)



import threading
import logging

from koverj.common.client.koverj_client import KoverjClient
from koverj.common.model.test_result import TestResult


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class LocatorsLifecycle(metaclass=Singleton):

    def __init__(self, client: KoverjClient = None):
        self.storage = threading.local()
        self.storage.locators = []
        self.client = client or KoverjClient()

    def send_locators(self, build_id: int, test_name: str):
        test_result = TestResult(buildId=build_id, testName=test_name, locators=list(set(self.storage.locators)))
        if self.client.config.log_locators:
            logging.debug(test_result)

        if self.client.config.use_koverj:
            self.client.send_test_result(test_result)
        self.storage.locators.clear()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['koverj',
 'koverj.agent',
 'koverj.agent.selenium',
 'koverj.common',
 'koverj.common.client',
 'koverj.common.config',
 'koverj.common.model',
 'koverj.common.plugin',
 'koverj.common.service']

package_data = \
{'': ['*'],
 'koverj': ['koverj-browser-plugin/*',
            'koverj-browser-plugin/assets/*',
            'koverj-browser-plugin/assets/css/*',
            'koverj-browser-plugin/assets/js/*',
            'koverj-browser-plugin/mock/*',
            'koverj-browser-plugin/plugin/*']}

install_requires = \
['configparser>=5.0.0,<6.0.0',
 'future>=0.18.2,<0.19.0',
 'pytest>=3.8.0',
 'requests>=2.23.0,<3.0.0',
 'selene>=1.0.0a13,<2.0.0',
 'selenium==3.141',
 'uplink>=0.9.1,<0.10.0']

setup_kwargs = {
    'name': 'koverj',
    'version': '0.1.102',
    'description': 'Test coverage for selenium based tests',
    'long_description': "### Python (>=3.7) agent that allow to collect locators from Selenium based UI tests.\nInspired by SergeyPirogov koverj-java-agent.\nBrowser_driver instance (e.g. webdriver.Chrome()) has to be created and passed to a LocatorsWatchingDriver\n\n#### - Selenium usage:\n```\nchrome_driver = webdriver.Chrome(executable_path=executable_path, options=chrome_options)\ndriver = LocatorsWatchingDriver(driver=chrome_driver)\n```\n\n#### - Selene usage:\n```\ndriver = LocatorsWatchingDriver(driver=chrome_driver)\nbrowser.set_driver(driver)\n```\n\nAn web_driver may be created as pytest fixture, e.g.:\n\n```\n@pytest.fixture()    \ndef web_driver(chrome_driver):\n    driver = LocatorsWatchingDrive(driver=chrome_web_driver)\n    browser.set_driver(driver)\n    yield browser.driver()\n    browser.quit()\n```\n\nIf pytest is used the koverj plugin may be registered in conftest.py:\n```\npytest_plugins = [\n    'koverj.common.plugin.pytest_koverj_plugin',\n]\n```\n\n#### Supported integrations:\n- Selene\n- Selenium\n\n#### Supported test runners:\n- pytest\n\nIn order to collect data you have to run koverj server (thanks to):\ndocker run -p 8086:8086 spirogov/koverj:0.1.0\n\nIn order to see results in your browser add **koverj-browser-plugin** to your browser.",
    'author': 'Ivan Huryn',
    'author_email': 'igur007@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Igur007/koverj-py',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support.event_firing_webdriver import EventFiringWebDriver
from koverj.agent.selenium.webdriver_locator_listener import WebDriverLocatorListener


class LocatorsWatchingDriver(EventFiringWebDriver):
    def __init__(self, driver: WebDriver):
        super().__init__(driver=driver, event_listener=WebDriverLocatorListener())
